@@ -31,7 +31,11 @@ and then write your t/*.t files like this:
 	   "Do they match?",
 	   This => $this,
 	   That => $that);
-    
+     
+    # That last one could have also been written... 
+    $T->ok_eq($this, $that);            # does 'eq' and logs operands
+    $T->ok_eqnum($this, $that);         # does '==' and logs operands 
+     
     # End testing:
     $T->end;   
 
@@ -81,9 +85,10 @@ A blank line follows each test's record, for clarity.
 use strict;
 use vars qw($VERSION);
 use FileHandle;
+use File::Basename;
 
 # The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.107 $, 10;
+$VERSION = substr q$Revision: 1.111 $, 10;
 
 
 #------------------------------
@@ -111,6 +116,7 @@ sub new {
 	Count=>0,
     }, shift;
     $self->log_open(@_) if @_;
+    $self;
 }
 
 #------------------------------
@@ -126,10 +132,12 @@ the output log file.
 
 sub typical {
     my $class = shift;
-    my ($tfile) = ($0 =~ m{([^/]+)\Z});
-    (-d "testout") 
-	or (mkdir "testout", 0755) 
+    my ($tfile) = basename $0;
+    unless (-d "testout") {
+	mkdir "testout", 0755 
 	    or die "Couldn't create a ./testout directory: $!\n";
+	### warn "$class: created 'testout' directory\n";
+    }
     $class->new("testout/${tfile}log");
 }
 
@@ -247,9 +255,15 @@ sub ok {
 	my ($k, $v) = (shift @ps, shift @ps);
 	my @vs = ((ref($v) and (ref($v) eq 'ARRAY'))? @$v : ($v));
 	foreach (@vs) { 
-	    s{([\n\t\x00-\x1F\x7F-\xFF\\"])}{'\\'.sprintf("%02X",ord($1)) }exg;
-	    s{\\0A}{\\n}g;
-	    $self->ln_print(qq{  $k: "$_"\n});
+	    if (!defined($_)) {  # value not defined: output keyword
+		$self->ln_print(qq{  $k: undef\n});
+	    }
+	    else {               # value defined: output quoted, encoded form
+		s{([\n\t\x00-\x1F\x7F-\xFF\\\"])}
+                 {'\\'.sprintf("%02X",ord($1)) }exg;
+	        s{\\0A}{\\n}g;
+	        $self->ln_print(qq{  $k: "$_"\n});
+            }
 	}
     }
     $self->ln_print($status, "\n");
@@ -257,6 +271,46 @@ sub ok {
     1;
 }
 
+
+#------------------------------
+
+=item ok_eq ASTRING, BSTRING, [TESTNAME], [PARAMHASH...]
+
+I<Instance method.>  
+Convenience front end to ok(): test whether C<ASTRING eq BSTRING>, and
+logs the operands as 'A' and 'B'.
+
+=cut
+
+sub ok_eq {
+    my ($self, $this, $that, $test, @ps) = @_;
+    $self->ok(($this eq $that), 
+	      ($test || "(Is 'A' string-equal to 'B'?)"),
+	      A => $this,
+	      B => $that,
+	      @ps);
+}
+
+
+#------------------------------
+
+=item ok_eqnum ANUM, BNUM, [TESTNAME], [PARAMHASH...]
+
+I<Instance method.>  
+Convenience front end to ok(): test whether C<ANUM == BNUM>, and
+logs the operands as 'A' and 'B'.  
+
+=cut
+
+sub ok_eqnum {
+    my ($self, $this, $that, $test, @ps) = @_;
+    $self->ok(($this == $that), 
+	      ($test || "(Is 'A' numerically-equal to 'B'?)"),
+	      A => $this,
+	      B => $that,
+	      @ps);
+}
+
 #------------------------------
 
 =back
@@ -362,9 +416,26 @@ sub ln_print {
 
 =back
 
-=head1 VERSION
 
-Revision: $Revision: 1.107 $
+=head1 CHANGE LOG
+
+B<Current version:>
+$Id: TBone.pm,v 1.111 1999/04/18 04:07:28 eryq Exp $
+
+=over 4
+
+
+=item Version 1.111
+
+Now uses File::Basename to create "typical" logfile name,
+for portability.
+
+
+=item Version 1.110
+
+Fixed bug in constructor that surfaced if no log was being used. 
+
+=back
 
 Created: Friday-the-13th of February, 1998.
 
